// Include required header files
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include "main.h"
#include "protocol.h"

// Declare global variables
volatile unsigned char rxBuffer[FRAME_SIZE];
volatile unsigned char rxBufferIndex = 0;

void USART_init() {
  // Setup USART1

  // UBRR1 is the (16-bit) register for specifying the baud rate
  UBRR1 = 51; // 51 = 9600 baud, see datasheet
  
  // UCSR1A is a control register for the USART
  //   MPCM1 enables the MPCM circuitry
  UCSR1A |= (1<<MPCM1);
  // UCSR1B is a control register for the USART
  //   RXEN1 enables the RX circuitry
  //   RxCIE1 enables the RX-done interrupt
  //   UCSZ12 is the third bit that determines the serial frame length
  UCSR1B |= (1<<RXEN1)|(1<<RXCIE1)|(1<<UCSZ12);
  // UCSR1C is a control register for the USART
  //   UCSZ11 is the second bit that determines the serial frame length
  //   UCSZ10 is the first bit that determines the serial frame length
  UCSR1C |= (1<<UCSZ11)|(1<<UCSZ10);
  // UCSZ1n = 111 means a serial frame length of 9 bits
}

void LED_init() {
  // DDRB is the data direction register of port B
  //   A "1" means the IO pin is an output
  DDRB |= (1<<DDB0);
  // PORTB is the value register of port B
  //   A "1" means the IO pin is driven high
  PORTB &= (0<<PB0);

  // DDRE is the data direction register of port E
  //   A "1" means the IO pin is an output
  DDRE |= (1<<DDE6);
  // PORTE is the value register of port E
  //   A "1" means the IO pin is driven high
  PORTE &= (0<<PE6);
}

int main(void) {
  // Run the main program

  // Initialize the peripherals
  USART_init();
  LED_init();

  // Enable global interrupts
  sei();

  // Main loop
  while(1) {
    if (rxBufferIndex == FRAME_SIZE) {
      // A full frame has been received
      switch (rxBuffer[0]) {
        // Process the command byte
        case 0xAA:
          // Command for LED received
          if (rxBuffer[1] == ON) {
            PORTB |= (1<<PB0);
          } else {
            PORTB &= ~(1<<PB0);
          }
          break; // never forget the break in a switch statement...
        default:
          break;
      }
      // Done processing the frame, reset the index for the next frame
      rxBufferIndex = 0;
    }
  }
}

ISR(USART1_RX_vect) {
  // Interrupt vector for TX ready
  //   This interrupt fires when a full byte has been received and loaded into the buffer

  unsigned char data;
  // Record the received byte from the buffer
  data = UDR1;

  //  Use MPCM-bit to check if we are in MPCM mode
  if ((UCSR1A & (1<<MPCM1)) == 1) {
    // Expecting an address frame
    if ((data == RX_UNICAST_ADDRESS) || (data == RX_BROADCAST_ADDRESS)) {
      // Our address or the broadcast address has been received
      PORTE |= (1<<PE6); // this is to indicate we received our address frame
      // Disable the MPCM mode to receive the data frame
      UCSR1A &= ~(1<<MPCM1);
    } else {
      // Not our address, remain in MPCM mode
    }
  } else {
    // Expecting a data frame
    PORTE &= ~(1<<PE6); // this is to indicate we received a data frame
    if ((UCSR1A & ((1 << FE1) | (1 << DOR1))) == 0) {
      // No errors detected, store the byte in the buffer
      rxBuffer[rxBufferIndex] = data;
      rxBufferIndex++;
      if (rxBufferIndex >= FRAME_SIZE) {
        // Complete frame received, go back to MPCM mode
        UCSR1A |= (1<<MPCM1);
        // Leave the index in a defined state to signal the complete frame
        rxBufferIndex = FRAME_SIZE;
      }
    }
  }
}
